@@ -4,51 +4,65 @@
 ##
 ###############################################################################
 
-package MCE::Shared::Base;
-
-use 5.010001;
 use strict;
 use warnings;
 
+use 5.010001;
+
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.006_02';
+package MCE::Shared::Base;
+
+our $VERSION = '1.867_01';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
-## do not remove numeric from no warnings above
+## no critic (Subroutines::ProhibitExplicitReturnUndef)
 
 use Scalar::Util qw( looks_like_number );
-use bytes;
 
-##  o Find feature
 ##
-##  o Used by MCE::Shared::{ Array, Hash, Minidb, and Ordhash }
-##    methods which take a query string for an argument.
+#  Several methods in MCE::Shared::{ Array, Cache, Hash, Minidb, and Ordhash }
+#  take a query string for an argument. The format of the string is described
+#  below. The _compile function is where the query string is evaluated and
+#  expanded into Perl code.
+# 
+#  In the context of sharing, the query mechanism is beneficial for the
+#  shared-manager process. The shared-manager runs the query where the data
+#  resides versus sending data in whole to the client process for traversing.
+#  Only the data found is sent back.
+# 
+#  o Basic demonstration
+# 
+#    @keys = $oh->keys( "query string given here" );
+#    @keys = $oh->keys( "val =~ /pattern/" );
+# 
+#  o Supported operators: =~ !~ eq ne lt le gt ge == != < <= > >=
+#  o Multiple expressions delimited by :AND or :OR, mixed case allowed
+# 
+#    "key eq 'some key' :or (val > 5 :and val < 9)"
+#    "key eq some key :or (val > 5 :and val < 9)"
+#    "key =~ /pattern/i :And field =~ /pattern/i"
+#    "key =~ /pattern/i :And index =~ /pattern/i"
+#    "index eq 'foo baz' :OR key !~ /pattern/i"    # 9 eq 'foo baz'
+#    "index eq foo baz :OR key !~ /pattern/i"      # 9 eq foo baz
+# 
+#    MCE::Shared::{ Array, Cache, Hash, Ordhash }
+#    * key matches on keys in the hash or index in the array
+#    * likewise, val matches on values
+# 
+#    MCE::Shared::{ Minidb }
+#    * key   matches on primary keys in the hash (H)oH or (H)oA
+#    * field matches on HoH->{key}{field} e.g. address
+#    * index matches on HoA->{key}[index] e.g. 9
+# 
+#  o Quoting is optional inside the string
+#
+#    "key =~ /pattern/i :AND field eq 'foo bar'"   # address eq 'foo bar'
+#    "key =~ /pattern/i :AND field eq foo bar"     # address eq foo bar
+#
+#  o See respective module in section labeled SYNTAX for QUERY STRING
+#    for demonstrations
 ##
-##  o Basic demonstration: @keys = $oh->keys( "val =~ /pattern/" );
-##  o Supported operators: =~ !~ eq ne lt le gt ge == != < <= > >=
-##  o Multiple expressions delimited by :AND or :OR
-##  o Quoting optional inside the string
-##
-##    "key eq 'some key' :or (val > 5 :and val < 9)"
-##    "key eq some key :or (val > 5 :and val < 9)"
-##    "key =~ /pattern/i :and field =~ /pattern/i"
-##    "key =~ /pattern/i :and index =~ /pattern/i"
-##    "key =~ /pattern/i :and field eq 'foo bar'"   # address eq 'foo bar'
-##    "key =~ /pattern/i :and field eq foo bar"     # address eq foo bar
-##    "index eq 'foo baz' :or key !~ /pattern/i"    # 9 eq 'foo baz'
-##    "index eq foo baz :or key !~ /pattern/i"      # 9 eq foo baz
-##
-##    MCE::Shared::{ Array, Hash, Ordhash }
-##    * key matches on keys in the hash or index in the array
-##    * val matches on values
-##
-##    MCE::Shared::Minidb
-##    * key   matches on primary keys in the hash (H)oH or (H)oA
-##    * field matches on HoH->{key}{field} e.g. address
-##    * index matches on HoA->{key}[index] e.g. 9
-##
-##  o The modifiers :AND and :OR may be mixed case. e.g. :And
 
 sub _compile {
    my ( $query ) = @_;
@@ -113,23 +127,22 @@ sub _find_array {
 
    # wants keys
    if ( $params->{'getkeys'} ) {
-      eval qq{ map { ($q) ? (\$_) : () } 0 .. \$#{ \$data } };
+      eval qq{ map { ($q) ? (\$_) : () } 0 .. \@{ \$data } - 1 };
    }
-
    # wants values
    elsif ( $params->{'getvals'} ) {
-      eval qq{ map { ($q) ? (\$data->[\$_]) : () } 0 .. \$#{ \$data } };
+      eval qq{ map { ($q) ? (\$data->[\$_]) : () } 0 .. \@{ \$data } - 1 };
    }
-
    # wants pairs
    else {
-      eval qq{ map { ($q) ? (\$_ => \$data->[\$_]) : () } 0 .. \$#{ \$data } };
+      eval qq{ map { ($q) ? (\$_ => \$data->[\$_]) : () } 0 .. \@{ \$data } - 1 };
    }
 }
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
-## Find items in HASH. Called by MCE::Shared::{ Hash, Minidb, Ordhash }.
+## Find items in HASH.
+## Called by MCE::Shared::{ Cache, Hash, Minidb, Ordhash }.
 ##
 ###############################################################################
 
@@ -144,7 +157,7 @@ sub _find_hash {
    $q =~ s/(!ref)\(key\)/$1(\$_)/gi;
 
    # Minidb (HoH) field
-   if ( $params->{'hfind'} ) {
+   if ( exists $params->{'hfind'} ) {
       $q =~ s/\$_ /:%: /g;  # preserve $_ from hash key mods above
       $q =~ s/([^:%\(\t ]+)[ ]+(==|!=|<|<=|>|>=|eq|ne|lt|le|gt|ge|=~|!~)/\$data->{\$_}{'$1'} $2/gi;
       $q =~ s/:%: /\$_ /g;  # restore hash key mods
@@ -153,7 +166,7 @@ sub _find_hash {
    }
 
    # Minidb (HoA) field
-   elsif ( $params->{'lfind'} ) {
+   elsif ( exists $params->{'lfind'} ) {
       $q =~ s/\$_ /:%: /g;  # preserve $_ from hash key mods above
       $q =~ s/([^:%\(\t ]+)[ ]+(==|!=|<|<=|>|>=|eq|ne|lt|le|gt|ge|=~|!~)/\$data->{\$_}['$1'] $2/gi;
       $q =~ s/:%: /\$_ /g;  # restore hash key mods
@@ -161,7 +174,7 @@ sub _find_hash {
       $q =~ s/(!ref)\(([^\$\)]+)\)/$1(\$data->{\$_}['$2'])/gi;
    }
 
-   # Hash/Ordhash value
+   # Cache/Hash/Ordhash value
    elsif ( $params->{'getvals'} && $q !~ /\(\$_/ ) {
       $grepvals = 1;
       $q =~ s/val[ ]+(==|!=|<|<=|>|>=|eq|ne|lt|le|gt|ge|=~|!~)/\$_ $1/gi;
@@ -180,19 +193,29 @@ sub _find_hash {
 
    # wants keys
    if ( $params->{'getkeys'} ) {
-      eval qq{ map { ($q) ? (\$_) : () } \$obj->keys };
+      eval qq{
+         map { ($q) ? (\$_) : () }
+            ( \$obj ? \$obj->keys : CORE::keys \%{\$data} )
+      };
    }
-
    # wants values
    elsif ( $params->{'getvals'} ) {
       $grepvals
-         ? eval qq{ grep { ($q) } \$obj->vals }
-         : eval qq{  map { ($q) ? (\$data->{\$_}) : () } \$obj->keys };
+         ? eval qq{
+              grep { ($q) }
+                 ( \$obj ? \$obj->vals : CORE::values \%{\$data} )
+           }
+         : eval qq{
+              map { ($q) ? (\$data->{\$_}) : () }
+                 ( \$obj ? \$obj->keys : CORE::keys \%{\$data} )
+           };
    }
-
    # wants pairs
    else {
-      eval qq{ map { ($q) ? (\$_ => \$data->{\$_}) : () } \$obj->keys };
+      eval qq{
+         map { ($q) ? (\$_ => \$data->{\$_}) : () }
+            ( \$obj ? \$obj->keys : CORE::keys \%{\$data} )
+      };
    }
 }
 
@@ -205,46 +228,67 @@ sub _find_hash {
 sub _stringify { no overloading;    "$_[0]" }
 sub _numify    { no overloading; 0 + $_[0]  }
 
-# Croak and die handler.
+# Croak handler.
 
 sub _croak {
-   if (defined $MCE::VERSION) {
+   if ( $INC{'MCE.pm'} ) {
       goto &MCE::_croak;
    }
+   elsif ( $INC{'MCE::Signal.pm'} ) {
+      $SIG{__DIE__}  = \&MCE::Signal::_die_handler;
+      $SIG{__WARN__} = \&MCE::Signal::_warn_handler;
+
+      $\ = undef; goto &Carp::croak;
+   }
    else {
       require Carp unless $INC{'Carp.pm'};
-      $SIG{__DIE__} = \&_die;
-      goto &Carp::croak;
+
+      $\ = undef; goto &Carp::croak;
    }
 }
 
-sub _die {
-   if (!defined $^S || $^S) {
-      if ( ($INC{'threads.pm'} && threads->tid() != 0) ||
-            $ENV{'PERL_IPERL_RUNNING'}
-      ) {
-         # thread env or running inside IPerl, check stack trace
-         my $_t = Carp::longmess(); $_t =~ s/\teval [^\n]+\n$//;
-         if ( $_t =~ /^(?:[^\n]+\n){1,7}\teval / ||
-              $_t =~ /\n\teval [^\n]+\n\t(?:eval|Try)/ )
-         {
-            CORE::die(@_);
-         }
-      }
-      else {
-         # normal env, trust $^S
-         CORE::die(@_);
+###############################################################################
+## ----------------------------------------------------------------------------
+## Common API for MCE::Shared::{ Array, Cache, Hash, Minidb, Ordhash }.
+##
+###############################################################################
+
+package MCE::Shared::Base::Common;
+
+# pipeline ( [ func1, @args ], [ func2, @args ], ... )
+
+sub pipeline {
+   my $self = shift;
+   my $tmp; $tmp = pop if ( defined wantarray );
+
+   while ( @_ ) {
+      my $cmd = shift; next unless ( ref $cmd eq 'ARRAY' );
+      if ( my $code = $self->can(shift @{ $cmd }) ) {
+         $code->($self, @{ $cmd });
       }
    }
 
-   local $\ = undef;
-   print {*STDERR} $_[0] if defined $_[0];
+   if ( defined $tmp ) {
+      my $code;
+      return ( ref $tmp eq 'ARRAY' && ( $code = $self->can(shift @{ $tmp }) ) )
+         ? $code->($self, @{ $tmp })
+         : undef;
+   }
 
-   ($^O eq 'MSWin32')
-      ? CORE::kill('KILL', -$$, $$)
-      : CORE::kill('INT', -getpgrp);
+   return;
+}
 
-   CORE::exit($?);
+# pipeline_ex ( [ func1, @args ], [ func2, @args ], ... )
+
+sub pipeline_ex {
+   my $self = shift;
+   my $code;
+
+   map {
+      ( ref $_ eq 'ARRAY' && ( $code = $self->can(shift @{ $_ }) ) )
+         ? $code->($self, @{ $_ })
+         : undef;
+   } @_;
 }
 
 1;
@@ -263,7 +307,7 @@ MCE::Shared::Base - Base package for helper classes
 
 =head1 VERSION
 
-This document describes MCE::Shared::Base version 1.006_02
+This document describes MCE::Shared::Base version 1.867_01
 
 =head1 DESCRIPTION
 
