@@ -18,24 +18,20 @@
 
 package MCE::Shared::Ordhash;
 
-use 5.010001;
 use strict;
 use warnings;
 
+use 5.010001;
+
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.006_02';
+our $VERSION = '1.867_01';
 
 ## no critic (Subroutines::ProhibitExplicitReturnUndef)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
-use MCE::Shared::Base;
-use bytes;
-
-# for marking a key to be garbage collected later
-use constant {
-   _TOMBSTONE => undef,
-};
+use MCE::Shared::Base ();
+use base 'MCE::Shared::Base::Common';
 
 use constant {
    _DATA => 0,  # unordered data
@@ -51,9 +47,11 @@ use overload (
    q("")    => \&MCE::Shared::Base::_stringify,
    q(0+)    => \&MCE::Shared::Base::_numify,
    q(%{})   => sub {
-      $_[0]->[_HREF] //= do {
+      no overloading;
+      $_[0]->[_HREF] || do {
+         # no circular reference to original, therefore no memory leaks
          tie my %h, __PACKAGE__.'::_href', bless([ @{ $_[0] } ], __PACKAGE__);
-         \%h;
+         $_[0]->[_HREF] = \%h;
       };
    },
    fallback => 1
@@ -139,7 +137,7 @@ sub DELETE {
       return delete $data->{ $key };
    }
 
-   # check the middle key
+   # must be a key somewhere in-between
    my $off = delete $indx->{ $key } // do {
       return undef unless ( exists $data->{ $key } );
 
@@ -153,10 +151,10 @@ sub DELETE {
       delete $indx->{ $key };
    };
 
-   $keys->[ $off -= ${ $begi } ] = _TOMBSTONE;
+   $keys->[ $off -= ${ $begi } ] = undef;   # tombstone
 
-   # GC keys if 75% or more are tombstones
-   if ( ++${ $gcnt } >= ( @{ $keys } >> 2 ) * 3 ) {
+   # GC keys if gcnt:size ratio is greater than 2:3
+   if ( ++${ $gcnt } > @{ $keys } * 0.667 ) {
       my $i; $i = ${ $begi } = ${ $gcnt } = 0;
 
       for my $k ( @{ $keys } ) {
@@ -215,7 +213,7 @@ sub _fill_index {
 
    # from end of list
    if ( !exists $indx->{ $keys->[-1] } ) {
-      my $i = ${ $begi } + $#{ $keys };
+      my $i = ${ $begi } + @{ $keys } - 1;
 
       for my $k ( reverse @{ $keys } ) {
          $i--, next unless ( defined $k );
@@ -269,28 +267,16 @@ sub POP {
 # PUSH ( key, value [, key, value, ... ] )
 
 sub PUSH {
-   if ( @_ == 3 ) {
-      my ( $key, $data, $keys ) = ( $_[1], @{ $_[0] } );
+   my $self = shift;
+   my ( $data, $keys ) = @{ $self };
+   my $key;
 
-      $_[0]->delete( $key ) if ( exists $data->{ $key } );
-      push @{ $keys }, "$key";
-      $data->{ $key } = $_[2];
-
-      defined wantarray ? scalar keys %{ $data } : ();
+   while ( @_ ) {
+      $self->delete( $key ) if ( exists $data->{ $key = shift } );
+      $data->{ $key } = shift, push @{ $keys }, "$key";
    }
-   else {
-      my $self = shift;
-      my ( $data, $keys ) = @{ $self };
-      my $key;
 
-      while ( @_ ) {
-         $self->delete( $key ) if ( exists $data->{ $key = shift } );
-         push @{ $keys }, "$key";
-         $data->{ $key } = shift;
-      }
-
-      defined wantarray ? scalar keys %{ $data } : ();
-   }
+   defined wantarray ? scalar keys %{ $data } : ();
 }
 
 # SHIFT ( )
@@ -319,24 +305,13 @@ sub SHIFT {
 
 sub UNSHIFT {
    my $self = shift;
-   my ( $data, $keys ) = @{ $self };
+   my ( $data, $keys, $indx, $begi ) = @{ $self };
    my $key;
 
-   if ( @_ == 2 ) {
-      $self->delete( $key ) if ( exists $data->{ $key = $_[0] } );
-      ${ $self->[_BEGI] }-- if %{ $self->[_INDX] };
-      unshift @{ $keys }, "$key";
-      $data->{ $key } = $_[1];
-   }
-   else {
-      my ( $indx, $begi ) = @{ $self }[ _INDX, _BEGI ];
-      while ( @_ ) {
-         $self->delete( $key ) if ( exists $data->{ $key = $_[-2] } );
-         ${ $begi }-- if %{ $indx };
-         unshift @{ $keys }, "$key";
-         $data->{ $key } = pop;
-         pop;
-      }
+   while ( @_ ) {
+      $self->delete( $key ) if ( exists $data->{ $key = $_[-2] } );
+      $data->{ $key } = pop, pop, unshift @{ $keys }, "$key";
+      ${ $begi }-- if %{ $indx };
    }
 
    defined wantarray ? scalar keys %{ $data } : ();
@@ -351,14 +326,15 @@ sub SPLICE {
 
    $self->purge if %{ $self->[_INDX] };
 
-   my ( $key, @ret );
    my $size = scalar @{ $keys };
    my $len  = @_ ? shift : $size - $off;
+   my @ret;
 
    if ( $off >= $size ) {
       $self->push( @_ ) if @_;
    }
    elsif ( abs($off) <= $size ) {
+      local $_;
       if ( $len > 0 ) {
          $off = $off + @{ $keys } if ( $off < 0 );
          my @k = splice @{ $keys }, $off, $len;
@@ -600,7 +576,7 @@ sub mset {
 sub purge {
    my ( $data, $keys, $indx, $begi, $gcnt ) = @{ $_[0] };
 
-   # TOMBSTONES, in-place purging for minimum memory consumption.
+   # purge in-place for minimum memory consumption
 
    if ( ${ $gcnt } ) {
       my $i = 0;
@@ -802,7 +778,8 @@ sub len {
 
 # For on-demand hash-like dereferencing.
 
-package MCE::Shared::Ordhash::_href;
+package # hide from rpm
+   MCE::Shared::Ordhash::_href;
 
 sub TIEHASH { $_[1] }
 
@@ -822,217 +799,279 @@ MCE::Shared::Ordhash - An ordered hash class featuring tombstone deletion
 
 =head1 VERSION
 
-This document describes MCE::Shared::Ordhash version 1.006_02
-
-=head1 SYNOPSIS
-
-   # non-shared
-   use MCE::Shared::Ordhash;
-
-   my $oh = MCE::Shared::Ordhash->new( @pairs );
-
-   # shared
-   use MCE::Shared;
-
-   my $oh = MCE::Shared->ordhash( @pairs );
-
-   # oo interface
-   $val   = $oh->set( $key, $val );
-   $val   = $oh->get( $key );
-   $val   = $oh->delete( $key );              # del is an alias for delete
-   $bool  = $oh->exists( $key );
-   void   = $oh->clear();
-   $len   = $oh->len();                       # scalar keys %{ $oh }
-   $len   = $oh->len( $key );                 # length $oh->{ $key }
-   @pair  = $oh->pop();
-   $len   = $oh->push( @pairs );
-   @pair  = $oh->shift();
-   $len   = $oh->unshift( @pairs );
-   %pairs = $oh->splice( $offset, $length, @pairs );
-
-   $oh2   = $oh->clone( @keys );              # @keys is optional
-   $oh3   = $oh->flush( @keys );
-   $iter  = $oh->iterator( @keys );           # ($key, $val) = $iter->()
-   @keys  = $oh->keys( @keys );
-   %pairs = $oh->pairs( @keys );
-   @vals  = $oh->values( @keys );             # vals is an alias for values
-
-   $len   = $ha->assign( $key/$val pairs );   # equivalent to ->clear, ->mset
-   $cnt   = $oh->mdel( @keys );
-   @vals  = $oh->mget( @keys );
-   $bool  = $oh->mexists( @keys );            # true if all keys exists
-   $len   = $oh->mset( $key/$val pairs );     # merge is an alias for mset
-
-   @vals  = $oh->sort();                      # by val $a <=> $b default
-   @vals  = $oh->sort( "desc" );              # by val $b <=> $a
-   @vals  = $oh->sort( "alpha" );             # by val $a cmp $b
-   @vals  = $oh->sort( "alpha desc" );        # by val $b cmp $a
-
-   @vals  = $oh->sort( "key" );               # by key $a <=> $b
-   @vals  = $oh->sort( "key desc" );          # by key $b <=> $a
-   @vals  = $oh->sort( "key alpha" );         # by key $a cmp $b
-   @vals  = $oh->sort( "key alpha desc" );    # by key $b cmp $a
-
-   # search capability key/val { =~ !~ eq ne lt le gt ge == != < <= > >= }
-   # key/val means to match against actual key/val respectively
-
-   @keys  = $oh->keys( "key eq 'some key' :or (val > 5 :and val < 9)" );
-   @keys  = $oh->keys( "key eq some key :or (val > 5 :and val < 9)" );
-
-   @keys  = $oh->keys( "key =~ /$pattern/i" );
-   @keys  = $oh->keys( "key !~ /$pattern/i" );
-   @keys  = $oh->keys( "val =~ /$pattern/i" );
-   @keys  = $oh->keys( "val !~ /$pattern/i" );
-
-   %pairs = $oh->pairs( "key == $number" );
-   %pairs = $oh->pairs( "key != $number :and val > 100" );
-   %pairs = $oh->pairs( "key <  $number :or key > $number" );
-   %pairs = $oh->pairs( "val <= $number" );
-   %pairs = $oh->pairs( "val >  $number" );
-   %pairs = $oh->pairs( "val >= $number" );
-
-   @vals  = $oh->values( "key eq $string" );
-   @vals  = $oh->values( "key ne $string with space" );
-   @vals  = $oh->values( "key lt $string :or val =~ /$pat1|$pat2/" );
-   @vals  = $oh->values( "val le $string :and val eq 'foo bar'" );
-   @vals  = $oh->values( "val le $string :and val eq foo bar" );
-   @vals  = $oh->values( "val gt $string" );
-   @vals  = $oh->values( "val ge $string" );
-
-   # sugar methods without having to call set/get explicitly
-
-   $len   = $oh->append( $key, $string );     #   $val .= $string
-   $val   = $oh->decr( $key );                # --$val
-   $val   = $oh->decrby( $key, $number );     #   $val -= $number
-   $val   = $oh->getdecr( $key );             #   $val--
-   $val   = $oh->getincr( $key );             #   $val++
-   $val   = $oh->incr( $key );                # ++$val
-   $val   = $oh->incrby( $key, $number );     #   $val += $number
-   $old   = $oh->getset( $key, $new );        #   $o = $v, $v = $n, $o
+This document describes MCE::Shared::Ordhash version 1.867_01
 
 =head1 DESCRIPTION
 
+An ordered-hash helper class for use as a standalone or managed by
+L<MCE::Shared>.
+
 This module implements an ordered hash featuring tombstone deletion,
-inspired by the L<Hash::Ordered> module. An ordered hash means the key
-insertion order is preserved.
+inspired by L<Hash::Ordered>. An ordered hash is very much like a normal
+hash but with key insertion order preserved.
 
 It provides C<splice>, sorting, plus extra capabilities for use with
 L<MCE::Shared::Minidb>. Tombstone deletion is further optimized to not
 impact C<store>, C<push>, C<unshift>, and C<merge>. Tombstones are
 purged in-place for lesser memory consumption. In addition, C<pop> and
 C<shift> run optimally when an index is present. The optimization also
-applies to forward and reverse deletes.
+applies to forward and reverse deletes. The end result is achieving a
+new level of performance, for a pure-Perl ordered hash implementation.
 
-The end result is achieving a new level of performance, for a pure-Perl
-ordered hash implementation.
+=head1 SYNOPSIS
+
+ # non-shared or local construction for use by a single process
+
+ use MCE::Shared::Ordhash;
+
+ my $oh = MCE::Shared::Ordhash->new( @pairs );
+
+ # construction for sharing with other threads and processes
+
+ use MCE::Shared;
+
+ my $oh = MCE::Shared->ordhash( @pairs );
+
+ # hash-like dereferencing
+
+ my $val = $oh->{$key};
+ $oh->{$key} = $val;
+
+ %{$oh} = ();
+
+ # OO interface
+
+ if ( !defined ( $val = $oh->get("some_key") ) ) {
+    $val = $oh->set( some_key => "some_value" );
+ }
+
+ $val   = $oh->set( $key, $val );
+ $val   = $oh->get( $key );
+ $val   = $oh->delete( $key );              # del is an alias for delete
+ $bool  = $oh->exists( $key );
+ void   = $oh->clear();
+ $len   = $oh->len();                       # scalar keys %{ $oh }
+ $len   = $oh->len( $key );                 # length $oh->{ $key }
+ @pair  = $oh->pop();
+ $len   = $oh->push( @pairs );
+ @pair  = $oh->shift();
+ $len   = $oh->unshift( @pairs );
+ %pairs = $oh->splice( $offset, $length, @pairs );
+
+ $oh2   = $oh->clone( @keys );              # @keys is optional
+ $oh3   = $oh->flush( @keys );
+ $iter  = $oh->iterator( @keys );           # ($key, $val) = $iter->()
+ @keys  = $oh->keys( @keys );
+ %pairs = $oh->pairs( @keys );
+ @vals  = $oh->values( @keys );             # vals is an alias for values
+
+ $len   = $oh->assign( $key/$val pairs );   # equivalent to ->clear, ->mset
+ $cnt   = $oh->mdel( @keys );
+ @vals  = $oh->mget( @keys );
+ $bool  = $oh->mexists( @keys );            # true if all keys exists
+ $len   = $oh->mset( $key/$val pairs );     # merge is an alias for mset
+
+ @vals  = $oh->sort();                      # by val $a <=> $b default
+ @vals  = $oh->sort( "desc" );              # by val $b <=> $a
+ @vals  = $oh->sort( "alpha" );             # by val $a cmp $b
+ @vals  = $oh->sort( "alpha desc" );        # by val $b cmp $a
+
+ @vals  = $oh->sort( "key" );               # by key $a <=> $b
+ @vals  = $oh->sort( "key desc" );          # by key $b <=> $a
+ @vals  = $oh->sort( "key alpha" );         # by key $a cmp $b
+ @vals  = $oh->sort( "key alpha desc" );    # by key $b cmp $a
+
+ # included, sugar methods without having to call set/get explicitly
+
+ $len   = $oh->append( $key, $string );     #   $val .= $string
+ $val   = $oh->decr( $key );                # --$val
+ $val   = $oh->decrby( $key, $number );     #   $val -= $number
+ $val   = $oh->getdecr( $key );             #   $val--
+ $val   = $oh->getincr( $key );             #   $val++
+ $val   = $oh->incr( $key );                # ++$val
+ $val   = $oh->incrby( $key, $number );     #   $val += $number
+ $old   = $oh->getset( $key, $new );        #   $o = $v, $v = $n, $o
+
+ # pipeline, provides atomicity for shared objects, MCE::Shared v1.09+
+
+ @vals  = $oh->pipeline(                    # ( "a_a", "b_b", "c_c" )
+    [ "set", foo => "a_a" ],
+    [ "set", bar => "b_b" ],
+    [ "set", baz => "c_c" ],
+    [ "mget", qw/ foo bar baz / ]
+ );
+
+For normal hash behavior, the TIE interface is supported.
+
+ # non-shared or local construction for use by a single process
+
+ use MCE::Shared::Ordhash;
+
+ tie my %oh, "MCE::Shared::Ordhash", @pairs;
+ tie my %oh, "MCE::Shared::Ordhash";
+
+ # construction for sharing with other threads and processes
+ # the ordered option is needed to know to use MCE::Shared::Ordhash
+
+ use MCE::Shared;
+
+ tie my %oh, "MCE::Shared", { ordered => 1 }, @pairs;
+ tie my %oh, "MCE::Shared", ordered => 1;
+
+ # usage
+
+ my $val;
+
+ if ( !defined ( $val = $oh{some_key} ) ) {
+    $val = $oh{some_key} = "some_value";
+ }
+
+ $oh{some_key} = 0;
+
+ tied(%oh)->incrby("some_key", 20);
+ tied(%oh)->incrby(some_key => 20);
 
 =head1 SYNTAX for QUERY STRING
 
-Several methods in C<MCE::Shared::Ordhash> take a query string for an argument.
+Several methods take a query string for an argument. The format of the string
+is described below. In the context of sharing, the query mechanism is beneficial
+for the shared-manager process. It is able to perform the query where the data
+resides versus the client-process grep locally involving lots of IPC.
 
-   o Basic demonstration: @keys = $oh->keys( "val =~ /pattern/" );
-   o Supported operators: =~ !~ eq ne lt le gt ge == != < <= > >=
-   o Multiple expressions delimited by :AND or :OR
-   o Quoting optional inside the string
+ o Basic demonstration
 
-     "key eq 'some key' :or (val > 5 :and val < 9)"
-     "key eq some key :or (val > 5 :and val < 9)"
-     "key =~ /pattern/i :and val =~ /pattern/i"
-     "key =~ /pattern/i :and val eq 'foo bar'"   # val eq "foo bar"
-     "key =~ /pattern/i :and val eq foo bar"     # val eq "foo bar"
-     "val eq foo baz :or key !~ /pattern/i"
+   @keys = $oh->keys( "query string given here" );
+   @keys = $oh->keys( "val =~ /pattern/" );
 
-     * key matches on keys in the hash
-     * val matches on values
+ o Supported operators: =~ !~ eq ne lt le gt ge == != < <= > >=
+ o Multiple expressions delimited by :AND or :OR, mixed case allowed
 
-=over 3
+   "key eq 'some key' :or (val > 5 :and val < 9)"
+   "key eq some key :or (val > 5 :and val < 9)"
+   "key =~ /pattern/i :And val =~ /pattern/i"
+   "val eq foo baz :OR key !~ /pattern/i"
 
-=item * The modifiers C<:AND> and C<:OR> may be mixed case. e.g. C<:And>
+   * key matches on keys in the hash
+   * likewise, val matches on values
 
-=back
+ o Quoting is optional inside the string
+
+   "key =~ /pattern/i :AND val eq 'foo bar'"   # val eq "foo bar"
+   "key =~ /pattern/i :AND val eq foo bar"     # val eq "foo bar"
+
+Examples.
+
+ # search capability key/val: =~ !~ eq ne lt le gt ge == != < <= > >=
+ # key/val means to match against actual key/val respectively
+
+ @keys  = $oh->keys( "key eq 'some key' :or (val > 5 :and val < 9)" );
+ @keys  = $oh->keys( "key eq some key :or (val > 5 :and val < 9)" );
+
+ @keys  = $oh->keys( "key =~ /$pattern/i" );
+ @keys  = $oh->keys( "key !~ /$pattern/i" );
+ @keys  = $oh->keys( "val =~ /$pattern/i" );
+ @keys  = $oh->keys( "val !~ /$pattern/i" );
+
+ %pairs = $oh->pairs( "key == $number" );
+ %pairs = $oh->pairs( "key != $number :and val > 100" );
+ %pairs = $oh->pairs( "key <  $number :or key > $number" );
+ %pairs = $oh->pairs( "val <= $number" );
+ %pairs = $oh->pairs( "val >  $number" );
+ %pairs = $oh->pairs( "val >= $number" );
+
+ @vals  = $oh->vals( "key eq $string" );
+ @vals  = $oh->vals( "key ne $string with space" );
+ @vals  = $oh->vals( "key lt $string :or val =~ /$pat1|$pat2/" );
+ @vals  = $oh->vals( "val le $string :and val eq 'foo bar'" );
+ @vals  = $oh->vals( "val le $string :and val eq foo bar" );
+ @vals  = $oh->vals( "val gt $string" );
+ @vals  = $oh->vals( "val ge $string" );
 
 =head1 API DOCUMENTATION
 
 This module involves TIE when accessing the object via hash-like behavior.
 Both non-shared and shared instances are impacted if doing so. Although likely
-fast enough for many use cases, use the OO interface if better performance is
-desired.
+fast enough for many use cases, the OO interface is recommended for best
+performance.
 
-=over 3
+=head2 MCE::Shared::Ordhash->new ( key, value [, key, value, ... ] )
 
-=item new ( key, value [, key, value, ... ] )
+=head2 MCE::Shared->ordhash ( key, value [, key, value, ... ] )
 
 Constructs a new object, with an optional list of key-value pairs.
 
-   # non-shared
-   use MCE::Shared::Ordhash;
+ # non-shared or local construction for use by a single process
 
-   $oh = MCE::Shared::Ordhash->new( @pairs );
-   $oh = MCE::Shared::Ordhash->new( );
+ use MCE::Shared::Ordhash;
 
-   # shared
-   use MCE::Shared;
+ $oh = MCE::Shared::Ordhash->new( @pairs );
+ $oh = MCE::Shared::Ordhash->new( );
 
-   $oh = MCE::Shared->ordhash( @pairs );
-   $oh = MCE::Shared->ordhash( );
+ # construction for sharing with other threads and processes
 
-=item assign ( key, value [, key, value, ... ] )
+ use MCE::Shared;
+
+ $oh = MCE::Shared->ordhash( @pairs );
+ $oh = MCE::Shared->ordhash( );
+
+=head2 assign ( key, value [, key, value, ... ] )
 
 Clears the hash, then sets multiple key-value pairs and returns the number of
 keys stored in the hash. This is equivalent to C<clear>, C<mset>.
 
-   $len = $ha->assign( "key1" => "val1", "key2" => "val2" );
+ $len = $oh->assign( "key1" => "val1", "key2" => "val2" );  # 2
+ $len = %{$oh} = ( "key1" => "val1", "key2" => "val2" );    # 4
 
 API available since 1.007.
 
-=item clear
+=head2 clear
 
 Removes all key-value pairs from the hash.
 
-   $oh->clear;
-   %{$oh} = ();
+ $oh->clear;
+ %{$oh} = ();
 
-=item clone ( key [, key, ... ] )
+=head2 clone ( key [, key, ... ] )
 
 Creates a shallow copy, a C<MCE::Shared::Ordhash> object. It returns an exact
 copy if no arguments are given. Otherwise, the object includes only the given
 keys in the same order. Keys that do not exist in the hash will have the
 C<undef> value.
 
-   $oh2 = $oh->clone( "key1", "key2" );
-   $oh2 = $oh->clone;
+ $oh2 = $oh->clone( "key1", "key2" );
+ $oh2 = $oh->clone;
 
-=item delete ( key )
+=head2 delete ( key )
 
 Deletes and returns the value by given key or C<undef> if the key does not
 exists in the hash.
 
-   $val = $oh->delete( "some_key" );
-   $val = delete $oh->{ "some_key" };
+ $val = $oh->delete( "some_key" );
+ $val = delete $oh->{ "some_key" };
 
-=item del
+=head2 del
 
 C<del> is an alias for C<delete>.
 
-=item exists ( key )
+=head2 exists ( key )
 
 Determines if a key exists in the hash.
 
-   if ( $oh->exists( "some_key" ) ) { ... }
-   if ( exists $oh->{ "some_key" } ) { ... }
+ if ( $oh->exists( "some_key" ) ) { ... }
+ if ( exists $oh->{ "some_key" } ) { ... }
 
-=item flush ( key [, key, ... ] )
+=head2 flush ( key [, key, ... ] )
 
 Same as C<clone>. Though, clears all existing items before returning.
 
-=item get ( key )
+=head2 get ( key )
 
 Gets the value of a hash key or C<undef> if the key does not exists.
 
-   $val = $oh->get( "some_key" );
-   $val = $oh->{ "some_key" };
+ $val = $oh->get( "some_key" );
+ $val = $oh->{ "some_key" };
 
-=item iterator ( key [, key, ... ] )
+=head2 iterator ( key [, key, ... ] )
 
 Returns a code reference for iterating a list of key-value pairs stored in
 the hash when no arguments are given. Otherwise, returns a code reference for
@@ -1043,283 +1082,331 @@ The list of keys to return is set when the closure is constructed. Later keys
 added to the hash are not included. Subsequently, the C<undef> value is
 returned for deleted keys.
 
-   $iter = $oh->iterator;
-   $iter = $oh->iterator( "key1", "key2" );
+ $iter = $oh->iterator;
+ $iter = $oh->iterator( "key1", "key2" );
 
-   while ( my ( $key, $val ) = $iter->() ) {
-      ...
-   }
+ while ( my ( $key, $val ) = $iter->() ) {
+    ...
+ }
 
-=item iterator ( "query string" )
+=head2 iterator ( "query string" )
 
 Returns a code reference for iterating a list of key-value pairs that match
 the given criteria. It returns an empty list if the search found nothing.
 The syntax for the C<query string> is described above.
 
-   $iter = $oh->iterator( "val eq some_value" );
-   $iter = $oh->iterator( "key eq some_key :AND val =~ /sun|moon|air|wind/" );
-   $iter = $oh->iterator( "val eq sun :OR val eq moon :OR val eq foo" );
-   $iter = $oh->iterator( "key =~ /$pattern/" );
+ $iter = $oh->iterator( "val eq some_value" );
+ $iter = $oh->iterator( "key eq some_key :AND val =~ /sun|moon|air|wind/" );
+ $iter = $oh->iterator( "val eq sun :OR val eq moon :OR val eq foo" );
+ $iter = $oh->iterator( "key =~ /$pattern/" );
 
-   while ( my ( $key, $val ) = $iter->() ) {
-      ...
-   }
+ while ( my ( $key, $val ) = $iter->() ) {
+    ...
+ }
 
-=item keys ( key [, key, ...] )
+=head2 keys ( key [, key, ...] )
 
 Returns hash keys in the same insertion order when no arguments are given.
 Otherwise, returns the given keys in the same order. Keys that do not exist
 will have the C<undef> value. In scalar context, returns the size of the hash.
 
-   @keys = $oh->keys;
-   @keys = $oh->keys( "key1", "key2" );
-   $len  = $oh->keys;
+ @keys = $oh->keys( "key1", "key2" );
 
-=item keys ( "query string" )
+ @keys = $oh->keys;     # faster
+ @keys = keys %{$oh};   # involves TIE overhead
+
+ $len  = $oh->keys;     # ditto
+ $len  = keys %{$oh};
+
+=head2 keys ( "query string" )
 
 Returns only keys that match the given criteria. It returns an empty list
 if the search found nothing. The syntax for the C<query string> is described
 above. In scalar context, returns the size of the resulting list.
 
-   @keys = $oh->keys( "val eq some_value" );
-   @keys = $oh->keys( "key eq some_key :AND val =~ /sun|moon|air|wind/" );
-   @keys = $oh->keys( "val eq sun :OR val eq moon :OR val eq foo" );
-   $len  = $oh->keys( "key =~ /$pattern/" );
+ @keys = $oh->keys( "val eq some_value" );
+ @keys = $oh->keys( "key eq some_key :AND val =~ /sun|moon|air|wind/" );
+ @keys = $oh->keys( "val eq sun :OR val eq moon :OR val eq foo" );
+ $len  = $oh->keys( "key =~ /$pattern/" );
 
-=item len ( key )
+=head2 len ( key )
 
 Returns the size of the hash when no arguments are given. For the given key,
 returns the length of the value stored at key or the C<undef> value if the
 key does not exists.
 
-   $size = $oh->len;
-   $len  = $oh->len( "key1" );
-   $len  = length $oh->{ "key1" };
+ $size = $oh->len;
+ $len  = $oh->len( "key1" );
+ $len  = length $oh->{ "key1" };
 
-=item mdel ( key [, key, ... ] )
+=head2 mdel ( key [, key, ... ] )
 
 Deletes one or more keys in the hash and returns the number of keys deleted.
 A given key which does not exist in the hash is not counted.
 
-   $cnt = $oh->mdel( "key1", "key2" );
+ $cnt = $oh->mdel( "key1", "key2" );
 
-=item mexists ( key [, key, ... ] )
+=head2 mexists ( key [, key, ... ] )
 
 Returns a true value if all given keys exists in the hash. A false value is
 returned otherwise.
 
-   if ( $oh->mexists( "key1", "key2" ) ) { ... }
+ if ( $oh->mexists( "key1", "key2" ) ) { ... }
 
-=item mget ( key [, key, ... ] )
+=head2 mget ( key [, key, ... ] )
 
 Gets the values of all given keys. It returns C<undef> for keys which do not
 exists in the hash.
 
-   ( $val1, $val2 ) = $oh->mget( "key1", "key2" );
+ ( $val1, $val2 ) = $oh->mget( "key1", "key2" );
 
-=item mset ( key, value [, key, value, ... ] )
+=head2 mset ( key, value [, key, value, ... ] )
 
 Sets multiple key-value pairs in a hash and returns the number of keys stored
 in the hash.
 
-   $len = $oh->mset( "key1" => "val1", "key2" => "val2" );
+ $len = $oh->mset( "key1" => "val1", "key2" => "val2" );
 
-=item merge
+=head2 merge
 
 C<merge> is an alias for C<mset>.
 
-=item pairs ( key [, key, ... ] )
+=head2 pairs ( key [, key, ... ] )
 
 Returns key-value pairs in the same insertion order when no arguments are given.
 Otherwise, returns key-value pairs for the given keys in the same order. Keys
 that do not exist will have the C<undef> value. In scalar context, returns the
 size of the hash.
 
-   @pairs = $oh->pairs;
-   @pairs = $oh->pairs( "key1", "key2" );
-   $len   = $oh->pairs;
+ @pairs = $oh->pairs( "key1", "key2" );
 
-=item pairs ( "query string" )
+ @pairs = $oh->pairs;
+ $len   = $oh->pairs;
+
+=head2 pairs ( "query string" )
 
 Returns only key-value pairs that match the given criteria. It returns an
 empty list if the search found nothing. The syntax for the C<query string> is
 described above. In scalar context, returns the size of the resulting list.
 
-   @pairs = $oh->pairs( "val eq some_value" );
-   @pairs = $oh->pairs( "key eq some_key :AND val =~ /sun|moon|air|wind/" );
-   @pairs = $oh->pairs( "val eq sun :OR val eq moon :OR val eq foo" );
-   $len   = $oh->pairs( "key =~ /$pattern/" );
+ @pairs = $oh->pairs( "val eq some_value" );
+ @pairs = $oh->pairs( "key eq some_key :AND val =~ /sun|moon|air|wind/" );
+ @pairs = $oh->pairs( "val eq sun :OR val eq moon :OR val eq foo" );
+ $len   = $oh->pairs( "key =~ /$pattern/" );
 
-=item pop
+=head2 pipeline ( [ func1, @args ], [ func2, @args ], ... )
+
+Combines multiple commands for the object to be processed serially. For shared
+objects, the call is made atomically due to single IPC to the shared-manager
+process. The C<pipeline> method is fully C<wantarray>-aware and receives a list
+of commands and their arguments. In scalar or list context, it returns data
+from the last command in the pipeline.
+
+ @vals = $oh->pipeline(                     # ( "a_a", "b_b", "c_c" )
+    [ "set", foo => "a_a" ],
+    [ "set", bar => "b_b" ],
+    [ "set", baz => "c_c" ],
+    [ "mget", qw/ foo bar baz / ]
+ );
+
+ $len = $oh->pipeline(                      # 3, same as $oh->len
+    [ "set", foo => "i_i" ],
+    [ "set", bar => "j_j" ],
+    [ "set", baz => "k_k" ],
+    [ "len" ]
+ );
+
+ $oh->pipeline(
+    [ "set", foo => "m_m" ],
+    [ "set", bar => "n_n" ],
+    [ "set", baz => "o_o" ]
+ );
+
+Current API available since 1.809.
+
+=head2 pipeline_ex ( [ func1, @args ], [ func2, @args ], ... )
+
+Same as C<pipeline>, but returns data for every command in the pipeline.
+
+ @vals = $oh->pipeline_ex(                  # ( "a_a", "b_b", "c_c" )
+    [ "set", foo => "a_a" ],
+    [ "set", bar => "b_b" ],
+    [ "set", baz => "c_c" ]
+ );
+
+Current API available since 1.809.
+
+=head2 pop
 
 Removes and returns the last key-value pair or value in scalar context of the
 ordered hash. If there are no keys in the hash, returns the undefined value.
 
-   ( $key, $val ) = $oh->pop;
+ ( $key, $val ) = $oh->pop;
 
-   $val = $oh->pop;
+ $val = $oh->pop;
 
-=item purge
+=head2 purge
 
 A utility method for purging any *tombstones* in the keys array. It also
 resets a couple counters internally. Call this method before serializing
 to a file, which is the case in C<MCE::Shared::Minidb>.
 
-   $oh->purge;
+ $oh->purge;
 
-=item push ( key, value [, key, value, ... ] )
+=head2 push ( key, value [, key, value, ... ] )
 
 Appends one or multiple key-value pairs to the tail of the ordered hash and
 returns the new length. Any keys already existing in the hash are re-inserted
 with the new values.
 
-   $len = $oh->push( "key1", "val1", "key2", "val2" );
+ $len = $oh->push( "key1", "val1", "key2", "val2" );
 
-=item set ( key, value )
+=head2 set ( key, value )
 
 Sets the value of the given hash key and returns its new value.
 
-   $val = $oh->set( "key", "value" );
-   $val = $oh->{ "key" } = "value";
+ $val = $oh->set( "key", "value" );
+ $val = $oh->{ "key" } = "value";
 
-=item shift
+=head2 shift
 
 Removes and returns the first key-value pair or value in scalar context of the
 ordered hash. If there are no keys in the hash, returns the undefined value.
 
-   ( $key, $val ) = $oh->shift;
+ ( $key, $val ) = $oh->shift;
 
-   $val = $oh->shift;
+ $val = $oh->shift;
 
-=item sort ( "BY key [ ASC | DESC ] [ ALPHA ]" )
+=head2 sort ( "BY key [ ASC | DESC ] [ ALPHA ]" )
 
-=item sort ( "BY val [ ASC | DESC ] [ ALPHA ]" )
+=head2 sort ( "BY val [ ASC | DESC ] [ ALPHA ]" )
 
 Returns sorted keys in list context, leaving the elements intact. In void
 context, sorts the hash in-place. By default, sorting is numeric and applied
 to values when no arguments are given.
 
-   @keys = $oh->sort( "BY val" );
+ @keys = $oh->sort( "BY val" );
 
-   $oh->sort();
+ $oh->sort();
 
 If the keys or values contain string values and you want to sort them
 lexicographically, specify the C<ALPHA> modifier.
 
-   @keys = $oh->sort( "BY key ALPHA" );
+ @keys = $oh->sort( "BY key ALPHA" );
 
-   $oh->sort( "BY val ALPHA" );
+ $oh->sort( "BY val ALPHA" );
 
 The default is C<ASC> for sorting the hash from small to large. In order to
 sort the hash from large to small, specify the C<DESC> modifier.
 
-   @keys = $oh->sort( "BY val DESC ALPHA" );
+ @keys = $oh->sort( "BY val DESC ALPHA" );
 
-   $oh->sort( "BY key DESC ALPHA" );
+ $oh->sort( "BY key DESC ALPHA" );
 
-=item splice ( offset [, length [, key, value, ... ] ] )
+=head2 splice ( offset [, length [, key, value, ... ] ] )
 
 Removes the key-value pairs designated by C<offset> and C<length> from the
 ordered hash, and replaces them with C<key-value pairs>, if any. The behavior
 is similar to the Perl C<splice> function.
 
-   @pairs = $oh->splice( 20, 2, @pairs );
-   @pairs = $oh->splice( 20, 2 );
-   @pairs = $oh->splice( 20 );
+ @pairs = $oh->splice( 20, 2, @pairs );
+ @pairs = $oh->splice( 20, 2 );
+ @pairs = $oh->splice( 20 );
 
-=item unshift ( key, value [, key, value, ... ] )
+=head2 unshift ( key, value [, key, value, ... ] )
 
 Prepends one or multiple key-value pairs to the head of the ordered hash and
 returns the new length. Any keys already existing in the hash are re-inserted
 with the new values.
 
-   $len = $oh->unshift( "key1", "val1", "key2", "val2" );
+ $len = $oh->unshift( "key1", "val1", "key2", "val2" );
 
-=item values ( key [, key, ... ] )
+=head2 values ( key [, key, ... ] )
 
 Returns hash values in the same insertion order when no arguments are given.
 Otherwise, returns values for the given keys in the same order. Keys that do
 not exist will have the C<undef> value. In scalar context, returns the size
 of the hash.
 
-   @vals = $oh->values;
-   @vals = $oh->values( "key1", "key2" );
-   $len  = $oh->values;
+ @vals = $oh->values( "key1", "key2" );
 
-=item values ( "query string" )
+ @vals = $oh->values;     # faster
+ @vals = values %{$oh};   # involves TIE overhead
+
+ $len  = $oh->values;     # ditto
+ $len  = values %{$oh};
+
+=head2 values ( "query string" )
 
 Returns only values that match the given criteria. It returns an empty list
 if the search found nothing. The syntax for the C<query string> is described
 above. In scalar context, returns the size of the resulting list.
 
-   @vals = $oh->values( "val eq some_value" );
-   @vals = $oh->values( "key eq some_key :AND val =~ /sun|moon|air|wind/" );
-   @vals = $oh->values( "val eq sun :OR val eq moon :OR val eq foo" );
-   $len  = $oh->values( "key =~ /$pattern/" );
+ @vals = $oh->values( "val eq some_value" );
+ @vals = $oh->values( "key eq some_key :AND val =~ /sun|moon|air|wind/" );
+ @vals = $oh->values( "val eq sun :OR val eq moon :OR val eq foo" );
+ $len  = $oh->values( "key =~ /$pattern/" );
 
-=item vals
+=head2 vals
 
 C<vals> is an alias for C<values>.
 
-=back
-
 =head1 SUGAR METHODS
 
 This module is equipped with sugar methods to not have to call C<set>
-and C<get> explicitly. The API resembles a subset of the Redis primitives
+and C<get> explicitly. In shared context, the benefit is atomicity and
+reduction in inter-process communication.
+
+The API resembles a subset of the Redis primitives
 L<http://redis.io/commands#strings> with key representing the hash key.
 
-=over 3
-
-=item append ( key, string )
+=head2 append ( key, string )
 
 Appends a value to a key and returns its new length.
 
-   $len = $oh->append( $key, "foo" );
+ $len = $oh->append( $key, "foo" );
 
-=item decr ( key )
+=head2 decr ( key )
 
 Decrements the value of a key by one and returns its new value.
 
-   $num = $oh->decr( $key );
+ $num = $oh->decr( $key );
 
-=item decrby ( key, number )
+=head2 decrby ( key, number )
 
 Decrements the value of a key by the given number and returns its new value.
 
-   $num = $oh->decrby( $key, 2 );
+ $num = $oh->decrby( $key, 2 );
 
-=item getdecr ( key )
+=head2 getdecr ( key )
 
 Decrements the value of a key by one and returns its old value.
 
-   $old = $oh->getdecr( $key );
+ $old = $oh->getdecr( $key );
 
-=item getincr ( key )
+=head2 getincr ( key )
 
 Increments the value of a key by one and returns its old value.
 
-   $old = $oh->getincr( $key );
+ $old = $oh->getincr( $key );
 
-=item getset ( key, value )
+=head2 getset ( key, value )
 
 Sets the value of a key and returns its old value.
 
-   $old = $oh->getset( $key, "baz" );
+ $old = $oh->getset( $key, "baz" );
 
-=item incr ( key )
+=head2 incr ( key )
 
 Increments the value of a key by one and returns its new value.
 
-   $num = $oh->incr( $key );
+ $num = $oh->incr( $key );
 
-=item incrby ( key, number )
+=head2 incrby ( key, number )
 
 Increments the value of a key by the given number and returns its new value.
 
-   $num = $oh->incrby( $key, 2 );
-
-=back
+ $num = $oh->incrby( $key, 2 );
 
 =head1 CREDITS
 
@@ -1334,105 +1421,116 @@ consumption. The closest module on CPAN to pass in this regard is
 L<Hash::Ordered> by David Golden.
 
 MCE::Shared has only one shared-manager process which is by design. Therefore,
-refactored tombstone deletion with extras for lesser impact to the rest of the
+re-factored tombstone deletion with extras for lesser impact to the rest of the
 library. This module differs in personality from Hash::Ordered mainly for
 compatibility with other classes included with MCE::Shared.
 
 The following simulates a usage pattern inside L<MCE::Hobo> involving random
 key deletion. For example, an application joining a list of Hobos provided by
-C<MCE::Hobo->list_joinable>.
+C<< MCE::Hobo->list_joinable >>.
 
-   use MCE::Shared::Ordhash;
-   use List::Util 'shuffle';
-   use Time::HiRes 'time';
+ use MCE::Shared::Ordhash;
+ use List::Util 'shuffle';
+ use Time::HiRes 'time';
 
-   srand 1618;
+ srand 0;
 
-   my $oh = MCE::Shared::Ordhash->new();
-   my $num_keys = 200000;
-   my $start = time();
+ my $oh = MCE::Shared::Ordhash->new();
+ my $num_keys = 200000;
+ my $start = time();
 
-   $oh->set($_,$_) for 1 .. $num_keys;
+ $oh->set($_,$_) for 1 .. $num_keys;
 
-   for ( shuffle $oh->keys ) {
-      $oh->delete($_);
-   }
+ for ( shuffle $oh->keys ) {
+    $oh->delete($_);
+ }
 
-   printf "duration: %7.03f secs\n", time() - $start;
+ printf "duration: %7.03f secs\n", time() - $start;
 
 Both the runtime and memory consumption are captured for the demonstration.
-Result for MCE::Shared::Hash, an unordered hash, is included for comparison.
+Results are included for MCE::Shared::Hash (unordered hash) for comparison.
 
-   for ( shuffle $oh->keys ) { $oh->delete($_) }
+ for ( shuffle $oh->keys ) { $oh->delete($_) }
 
-   0.378 secs.  35 MB  MCE::Shared::Hash (unordered)
-   0.437 secs.  49 MB  Tie::Hash::Indexed (XS)
-   0.733 secs.  54 MB  MCE::Shared::Ordhash
-   1.065 secs.  60 MB  Hash::Ordered
-   1.752 secs. 112 MB  Tie::LLHash
-    > 42 mins.  66 MB  Tie::IxHash
+ 0.378 secs.  35 MB  MCE::Shared::Hash (unordered)
+ 0.437 secs.  49 MB  Tie::Hash::Indexed (XS)
+ 0.743 secs.  54 MB  MCE::Shared::Ordhash
+ 1.028 secs.  60 MB  Hash::Ordered
+ 1.752 secs. 112 MB  Tie::LLHash
+  > 42 mins.  66 MB  Tie::IxHash
 
 Using the same demonstration above, another usage pattern inside L<MCE::Hobo>
 involves orderly hash-key deletion. For example, waiting for and joining all
-Hobos provided by C<MCE::Hobo->list>.
+Hobos provided by C<< MCE::Hobo->list >>.
 
-   for ( $oh->keys ) { $oh->delete($_) }
+ for ( $oh->keys ) { $oh->delete($_) }
 
-   0.353 secs.  35 MB  MCE::Shared::Hash (unordered)
-   0.349 secs.  49 MB  Tie::Hash::Indexed (XS)
-   0.452 secs.  41 MB  MCE::Shared::Ordhash
-   0.797 secs.  54 MB  Hash::Ordered
-   1.338 secs. 112 MB  Tie::LLHash
-    > 42 mins.  66 MB  Tie::IxHash
+ 0.353 secs.  35 MB  MCE::Shared::Hash (unordered)
+ 0.349 secs.  49 MB  Tie::Hash::Indexed (XS)
+ 0.452 secs.  41 MB  MCE::Shared::Ordhash
+ 0.735 secs.  54 MB  Hash::Ordered
+ 1.338 secs. 112 MB  Tie::LLHash
+  > 42 mins.  66 MB  Tie::IxHash
 
 No matter if orderly or randomly, even backwards, hash-key deletion in
-C<MCE::Shared::Ordhash> performs reasonably well.
+C<MCE::Shared::Ordhash> performs reasonably well. The following provides
+the construction used for the modules mentioned.
 
-The following provides the construction used for the modules mentioned.
-Basically, key setting and deletion is through the OO interface for fair
-comparison.
+ my $oh = Hash::Ordered->new();
+    $oh->set($_,$_);   $oh->keys;  $oh->delete($_);
 
-   my $oh = Hash::Ordered->new();
-      $oh->set($_,$_);   $oh->keys;  $oh->delete($_);
+ my $oh = Tie::Hash::Indexed->new();
+    $oh->set($_,$_);   $oh->keys;  $oh->delete($_);
 
-   my $oh = Tie::Hash::Indexed->new();
-      $oh->set($_,$_);   $oh->keys;  $oh->delete($_);
+ my $oh = Tie::IxHash->new();
+    $oh->STORE($_,$_); $oh->Keys;  $oh->DELETE($_);
 
-   my $oh = Tie::IxHash->new();
-      $oh->STORE($_,$_); $oh->Keys;  $oh->DELETE($_);
+ my $oh = tie my %hash, 'Tie::LLHash';
+    $oh->last($_,$_);  keys %hash; $oh->DELETE($_);
 
-   my $oh = tie my %hash, 'Tie::LLHash';
-      $oh->last($_,$_);  keys %hash; $oh->DELETE($_);
+Hash::Ordered is supported for use with MCE::Shared. This includes on-demand
+hash-like dereferencing, similarly to C<hash> and C<ordhash>.
 
-Hash::Ordered is supported for use with MCE::Shared. The on-demand hash-like
-dereferencing is provided by MCE::Shared.
+ use feature 'say';
 
-   use feature 'say';
+ use MCE::Hobo;
+ use MCE::Shared;
+ use Hash::Ordered; # 0.010 or later
 
-   use MCE::Hobo;
-   use MCE::Shared;
+ my $ha = MCE::Shared->hash();    # shared MCE::Shared::Hash
+ my $oh = MCE::Shared->ordhash(); # shared MCE::Shared::Ordhash
 
-   use Hash::Ordered;  # 0.010 or later
+ my $ho = MCE::Shared->share( Hash::Ordered->new() );
 
-   my $ho = MCE::Shared->share( Hash::Ordered->new() );
+ sub parallel_task {
+    my ($id) = @_;
 
-   sub parallel_task {
-      my ($id) = @_;
+    # OO interface
+    if ($id == 1) {
+       $ha->set("$id", "foo");
+       $oh->set("$id", "foo");
+       $ho->set("$id", "foo");
+    }
+    # hash-like dereferencing
+    elsif ($id == 2) {
+       $ha->{"$id"} = "baz";
+       $oh->{"$id"} = "baz";
+       $ho->{"$id"} = "baz";
+    }
 
-      # OO interface
-      $ho->set("$id", "foo") if ($id == 1);
+    return;
+ }
 
-      # hash-like dereferencing
-      $ho->{"$id"} = "baz"   if ($id == 2);
+ MCE::Hobo->create("parallel_task", $_) for 1..2;
+ MCE::Hobo->waitall;
 
-      return;
-   }
+ say $ha->{"1"};     # foo
+ say $oh->{"1"};
+ say $ho->{"1"};
 
-   MCE::Hobo->create("parallel_task", $_) for 1..2;
-   MCE::Hobo->waitall;
-
-   say $ho->{"1"};     # foo
-   say $ho->get("2");  # baz
+ say $ha->get("2");  # baz
+ say $oh->get("2");
+ say $ho->get("2");
 
 =head1 SEE ALSO
 
