@@ -6,18 +6,18 @@
 
 package MCE::Shared::Scalar;
 
-use 5.010001;
 use strict;
 use warnings;
 
+use 5.010001;
+
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.006_02';
+our $VERSION = '1.867_01';
 
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
-use MCE::Shared::Base;
-use bytes;
+use MCE::Shared::Base ();
 
 use overload (
    q("")    => \&MCE::Shared::Base::_stringify,
@@ -100,147 +100,175 @@ MCE::Shared::Scalar - Scalar helper class
 
 =head1 VERSION
 
-This document describes MCE::Shared::Scalar version 1.006_02
-
-=head1 SYNOPSIS
-
-   # non-shared
-   use MCE::Shared::Scalar;
-
-   my $var = MCE::Shared::Scalar->new( $val );
-
-   # shared
-   use MCE::Shared;
-
-   my $var = MCE::Shared->scalar( $val );
-
-   # oo interface
-   $val = $var->set( $val );
-   $val = $var->get();
-   $len = $var->len();
-
-   # sugar methods without having to call set/get explicitly
-   $val = $var->append( $string );     #   $val .= $string
-   $val = $var->decr();                # --$val
-   $val = $var->decrby( $number );     #   $val -= $number
-   $val = $var->getdecr();             #   $val--
-   $val = $var->getincr();             #   $val++
-   $val = $var->incr();                # ++$val
-   $val = $var->incrby( $number );     #   $val += $number
-   $old = $var->getset( $new );        #   $o = $v, $v = $n, $o
+This document describes MCE::Shared::Scalar version 1.867_01
 
 =head1 DESCRIPTION
 
-Helper class for L<MCE::Shared>.
+A scalar helper class for use as a standalone or managed by L<MCE::Shared>.
+
+=head1 SYNOPSIS
+
+ # non-shared or local construction for use by a single process
+
+ use MCE::Shared::Scalar;
+
+ my $var = MCE::Shared::Scalar->new( $val );
+
+ # construction for sharing with other threads and processes
+
+ use MCE::Shared;
+
+ my $var = MCE::Shared->scalar( $val );
+
+ # scalar-like dereferencing
+
+ my $val = ${ $var };
+ ${ $var } = $val;
+
+ # OO interface
+
+ $val = $var->set( $val );
+ $val = $var->get();
+ $len = $var->len();
+
+ # included, sugar methods without having to call set/get explicitly
+
+ $val = $var->append( $string );     #   $val .= $string
+ $val = $var->decr();                # --$val
+ $val = $var->decrby( $number );     #   $val -= $number
+ $val = $var->getdecr();             #   $val--
+ $val = $var->getincr();             #   $val++
+ $val = $var->incr();                # ++$val
+ $val = $var->incrby( $number );     #   $val += $number
+ $old = $var->getset( $new );        #   $o = $v, $v = $n, $o
+
+For normal scalar behavior, the TIE interface is supported.
+
+ # non-shared or local construction for use by a single process
+
+ use MCE::Shared::Scalar;
+
+ tie my $var, "MCE::Shared::Scalar";
+
+ # construction for sharing with other threads and processes
+
+ use MCE::Shared;
+
+ tie my $var, "MCE::Shared";
+
+ # usage
+
+ $var = 0;
+
+ tied($var)->incrby(20);
 
 =head1 API DOCUMENTATION
 
 This module may involve TIE when accessing the object via scalar dereferencing.
 Only shared instances are impacted if doing so. Although likely fast enough for
-many use cases, use the OO interface if better performance is desired.
+many use cases, the OO interface is recommended for best performance.
 
-=over 3
+=head2 MCE::Shared::Scalar->new ( [ value ] )
 
-=item new ( [ value ] )
+=head2 MCE::Shared->scalar ( [ value ] )
 
 Constructs a new object. Its value is undefined when C<value> is not specified.
 
-   # non-shared
-   use MCE::Shared::Scalar;
+ # non-shared or local construction for use by a single process
 
-   $var = MCE::Shared::Scalar->new( "foo" );
-   $var = MCE::Shared::Scalar->new;
+ use MCE::Shared::Scalar;
 
-   # shared
-   use MCE::Shared;
+ $var = MCE::Shared::Scalar->new( "foo" );
+ $var = MCE::Shared::Scalar->new;
 
-   $var = MCE::Shared->scalar( "bar" );
-   $var = MCE::Shared->scalar;
+ # construction for sharing with other threads and processes
 
-=item set ( value )
+ use MCE::Shared;
+
+ $var = MCE::Shared->scalar( "bar" );
+ $var = MCE::Shared->scalar;
+
+=head2 set ( value )
 
 Preferably, set the value via the OO interface. Otherwise, C<TIE> is activated
 on-demand for setting the value. The new value is returned in scalar context.
 
-   $val = $var->set( "baz" );
-   $var->set( "baz" );
-   ${$var} = "baz";
+ $val = $var->set( "baz" );
+ $var->set( "baz" );
+ ${$var} = "baz";
 
-=item get
+=head2 get
 
 Likewise, obtain the value via the OO interface. C<TIE> is utilized for
 retrieving the value otherwise.
 
-   $val = $var->get;
-   $val = ${$var};
+ $val = $var->get;
+ $val = ${$var};
 
-=item len
+=head2 len
 
 Returns the length of the value. It returns the C<undef> value if the value
 is not defined.
 
-   $len = $var->len;
-   length ${$var};
-
-=back
+ $len = $var->len;
+ length ${$var};
 
 =head1 SUGAR METHODS
 
 This module is equipped with sugar methods to not have to call C<set>
-and C<get> explicitly. The API resembles a subset of the Redis primitives
+and C<get> explicitly. In shared context, the benefit is atomicity and
+reduction in inter-process communication.
+
+The API resembles a subset of the Redis primitives
 L<http://redis.io/commands#strings> without the key argument.
 
-=over 3
-
-=item append ( value )
+=head2 append ( value )
 
 Appends a value at the end of the current value and returns its new length.
 
-   $len = $var->append( "foo" );
+ $len = $var->append( "foo" );
 
-=item decr
+=head2 decr
 
 Decrements the value by one and returns its new value.
 
-   $num = $var->decr;
+ $num = $var->decr;
 
-=item decrby ( number )
+=head2 decrby ( number )
 
 Decrements the value by the given number and returns its new value.
 
-   $num = $var->decrby( 2 );
+ $num = $var->decrby( 2 );
 
-=item getdecr
+=head2 getdecr
 
 Decrements the value by one and returns its old value.
 
-   $old = $var->getdecr;
+ $old = $var->getdecr;
 
-=item getincr
+=head2 getincr
 
 Increments the value by one and returns its old value.
 
-   $old = $var->getincr;
+ $old = $var->getincr;
 
-=item getset ( value )
+=head2 getset ( value )
 
 Sets the value and returns its old value.
 
-   $old = $var->getset( "baz" );
+ $old = $var->getset( "baz" );
 
-=item incr
+=head2 incr
 
 Increments the value by one and returns its new value.
 
-   $num = $var->incr;
+ $num = $var->incr;
 
-=item incrby ( number )
+=head2 incrby ( number )
 
 Increments the value by the given number and returns its new value.
 
-   $num = $var->incrby( 2 );
-
-=back
+ $num = $var->incrby( 2 );
 
 =head1 CREDITS
 
